@@ -21,7 +21,6 @@ use fields (
 	'domain2proxy',   # optional mapping between SIP domains and proxies (otherwise use DNS)
 	# internals
 	'do_retransmits', # flag if retransmits will be done (false for stateless proxy)
-	'outgoing_leg',   # Leg for outgoing_proxy
 	'queue',          # \@list of outstanding Net::SIP::Dispatcher::Packet
 	'response_cache', # Cache of responses, used to reply to retransmits
 	'disp_expire',    # expire/retransmit timer
@@ -84,7 +83,6 @@ sub new {
 		legs => [],
 		queue  => [],
 		outgoing_proxy => undef,
-		outgoing_leg => undef,
 		response_cache => {},
 		do_retransmits => defined( $do_retransmits ) ? $do_retransmits : 1,
 		eventloop => $eventloop,
@@ -97,7 +95,6 @@ sub new {
 		my $leg = $self->_find_leg4addr( $outgoing_proxy )
 			|| die "cannot find leg for destination $outgoing_proxy";
 		$self->{outgoing_proxy} = $outgoing_proxy;
-		$self->{outgoing_leg}   = $leg;
 	}
 
 
@@ -175,21 +172,54 @@ sub add_leg {
 
 		if ( my $fd = $leg->fd ) {
 			my $cb = sub {
-				my ($self,$leg) = @_;
-				$self || return;
+				# don't crash Dispatcher on bad or unexpected packets
+				eval {
+					my ($self,$leg) = @_;
+					$self || return;
+
+					# leg->receive might return undef if the packet wasnt
+					# read successfully. for tcp connections the receive
+					# on a listening socket might cause a new leg to be added
+					# which then will receive the packet (maybe over multiple
+					# read attempts)
+					my ($packet,$from) = $leg->receive or do {
+						DEBUG( 50,"failed to receive on leg" );
+						return;
+					};
+
+					if ($packet->is_request) {
+						# add received and rport to top via
+						$packet->scan_header( via => [ sub {
+							my ($vref,$hdr) = @_;
+							return if $$vref++;
+							my ($d,$h) = sip_hdrval2parts(via => $hdr->{value});
+							# FIXME: not IPv6 save
+							my ($host,$port) = $d =~m{^\S+\s+(\S+?)(?::(\d+))?$};
+							my ($addr,$rport) = $from =~m{^(\S+)(?::(\d+))$};
+							my %nh;
+							if ( exists $h->{rport} and ! defined $h->{rport}) {
+								$nh{rport} = $rport;
+							}
+							if ( $host ne $addr or $nh{rport}) { 
+								# either hostname or different IP or required because
+								# rport was set
+								$nh{received} = $addr;
+							}
+							if (%nh) {
+								$hdr->{value} = sip_parts2hdrval('via',$d,{ %$h,%nh});
+								$hdr->set_modified;
+							}
+						}, \( my $cvia )]);
+					}
+
+					# handle received packet
+					$self->receive( $packet,$leg,$from );
+
 
-				# leg->receive might return undef if the packet wasnt
-				# read successfully. for tcp connections the receive
-				# on a listening socket might cause a new leg to be added
-				# which then will receive the packet (maybe over multiple
-				# read attempts)
-				my ($packet,$from) = $leg->receive or do {
-					DEBUG( 50,"failed to receive on leg" );
-					return;
 				};
-
-				# handle received packet
-				$self->receive( $packet,$leg,$from );
+				if ($@) {
+					DEBUG(1,"dispatcher croaked: $@");
+				}
 			};
 			$cb = [ $cb,$self,$leg ];
 			weaken( $cb->[1] );
@@ -319,7 +349,7 @@ sub deliver {
 #   $typ: what to cancel, e.g. 'id','callid' or 'qentry', optional,
 #     defaults to 'id' if $id is not ref or 'qentry' if $id is ref
 #   $id: id to cancel, can also be queue entry
-# Returns: NONE
+# Returns: bool, true if the was something canceled
 ###########################################################################
 sub cancel_delivery {
 	my Net::SIP::Dispatcher $self = shift;
@@ -340,6 +370,7 @@ sub cancel_delivery {
 		}
 	}
 	my $q = $self->{queue};
+	my $qn = @$q;
 	if ( $qentry ) {
 		# it's a *::Dispatcher::Packet
 		DEBUG( 100,"cancel packet id: $qentry->{id}" );
@@ -355,6 +386,7 @@ sub cancel_delivery {
 	} else {
 		croak( "cancel_delivery w/o id" );
 	}
+	return @$q < $qn; # true if items got deleted
 }
 
 
@@ -986,7 +1018,7 @@ sub trigger_callback {
 	my Net::SIP::Dispatcher::Packet $self = shift;
 	my $error = shift;
 	my $cb = $self->{callback} || return;
-	invoke_callback( $cb, $error ? ($error,$self):() );
+	invoke_callback( $cb,$error,$self);
 	return 1;
 }
 
