@@ -130,19 +130,6 @@ sub forward_incoming {
 		}
 		$packet->set_header( 'max-forwards',$maxf );
 
-		# add received to top via
-		my $via;
-		$packet->scan_header( via => [ sub {
-			my ($vref,$hdr) = @_;
-			if ( !$$vref ) {
-				# XXXXXXX maybe check that no received header existed before
-				$$vref = $hdr->{value}.=
-					";received=$self->{addr}:$self->{port}";
-				$hdr->set_modified;
-			}
-		}, \$via ]);
-
-
 		# check if last hop was strict router
 		# remove myself from route
 		my $uri = $packet->uri;
@@ -263,7 +250,7 @@ sub forward_outgoing {
 #   $addr:   ip:port where to deliver
 #   $callback: optional callback, if an error occured the callback will
 #      be called with $! as argument. If no error occured and the
-#      proto is tcp the callback will be called with ENOERR to show
+#      proto is tcp the callback will be called with error=0 to show
 #      that the packet was definitly delivered (and need not retried)
 ###########################################################################
 sub deliver {
@@ -376,7 +363,7 @@ sub sendto {
 		return;
 	}
 
-	# XXXX dont forget to call callback back with ENOERR if
+	# XXXX dont forget to call callback back with error=0 if
 	# delivery by tcp successful
 	return 1;
 }
@@ -404,6 +391,13 @@ sub receive {
 		return;
 	};
 
+	# packet must be at least 13 bytes big (first line incl version
+	# + final crlf crlf). Ignore anything smaller, probably keep-alives
+	if ( length($buf)<13 ) {
+		DEBUG(11,"ignored packet with len ".length($buf)." because to small (keep-alive?)");
+		return;
+	}
+
 	my $packet = eval { Net::SIP::Packet->new( $buf ) } or do {
 		DEBUG( 3,"cannot parse buf as SIP: $@\n$buf" );
 		return;
