@@ -16,6 +16,70 @@ use Net::SIP::Util 'invoke_callback';
 
 use Digest::MD5 'md5_hex';
 
+my %ResponseCode = (
+	# Informational
+	100 => 'Trying',
+	180 => 'Ringing',
+	181 => 'Call Is Being Forwarded',
+	182 => 'Queued',
+	183 => 'Session Progress',
+
+	# Success
+	200 => 'OK',
+
+	# Redirection
+	300 => 'Multiple Choices',
+	301 => 'Moved Permanently',
+	302 => 'Moved Temporarily',
+	305 => 'Use Proxy',
+	380 => 'Alternative Service',
+
+	# Client-Error
+	400 => 'Bad Request',
+	401 => 'Unauthorized',
+	402 => 'Payment Required',
+	403 => 'Forbidden',
+	404 => 'Not Found',
+	405 => 'Method Not Allowed',
+	406 => 'Not Acceptable',
+	407 => 'Proxy Authentication Required',
+	408 => 'Request Timeout',
+	410 => 'Gone',
+	413 => 'Request Entity Too Large',
+	414 => 'Request-URI Too Large',
+	415 => 'Unsupported Media Type',
+	416 => 'Unsupported URI Scheme',
+	420 => 'Bad Extension',
+	421 => 'Extension Required',
+	423 => 'Interval Too Brief',
+	480 => 'Temporarily not available',
+	481 => 'Call Leg/Transaction Does Not Exist',
+	482 => 'Loop Detected',
+	483 => 'Too Many Hops',
+	484 => 'Address Incomplete',
+	485 => 'Ambiguous',
+	486 => 'Busy Here',
+	487 => 'Request Terminated',
+	488 => 'Not Acceptable Here',
+	491 => 'Request Pending',
+	493 => 'Undecipherable',
+
+	# Server-Error
+	500 => 'Internal Server Error',
+	501 => 'Not Implemented',
+	502 => 'Bad Gateway',
+	503 => 'Service Unavailable',
+	504 => 'Server Time-out',
+	505 => 'SIP Version not supported',
+	513 => 'Message Too Large',
+
+	# Global-Failure
+	600 => 'Busy Everywhere',
+	603 => 'Decline',
+	604 => 'Does not exist anywhere',
+	606 => 'Not Acceptable',
+);
+
 ###########################################################################
 # Redefine methods from Net::SIP::Packet, no need to find out dynamically
 ###########################################################################
@@ -98,9 +162,9 @@ sub create_cancel {
 
 ###########################################################################
 # Create response to request
-# Args: ($self,$code,$msg;$args,$body)
+# Args: ($self,$code,[$msg],[$args,$body])
 #   $code: numerical response code
-#   $msg:  text for response code
+#   $msg: msg for code, if arg not given it will be used from %ResponseCode
 #   $args: additional args for SIP header
 #   $body: body as string
 # Returns: $response
@@ -108,8 +172,9 @@ sub create_cancel {
 ###########################################################################
 sub create_response {
 	my Net::SIP::Request $self = shift;
-	my ($code,$msg,$args,$body) = @_;
-
+	my $code = shift;
+	my ($msg,$args,$body) = ( defined $_[0] && ref($_[0]) ) ? (undef,@_):@_;
+	$msg = $ResponseCode{$code} if ! defined $msg;
 	my %header = (
 		cseq      => scalar($self->get_header('cseq')),
 		'call-id' => scalar($self->get_header('call-id')),
@@ -185,9 +250,9 @@ sub authorize {
 
 				if ( lc($a->{data}) ne 'digest'
 					|| $h->{algorithm} && lc($h->{algorithm}) ne 'md5'
-					|| $h->{qop} && lc($h->{qop}) ne 'auth' ) {
+					|| $h->{qop} && $h->{qop} !~ m{(?:^|,\s*)auth(?:$|,)}i ) {
 					no warnings;
-					#warn "unsupported authorization method $a->{data} method=$h->{method} qop=$h->{qop}";
+					DEBUG(10,"unsupported authorization method $a->{data} method=$h->{method} qop=$h->{qop}");
 					next;
 				}
 				my $realm = $h->{realm};
@@ -211,6 +276,7 @@ sub authorize {
 
 				# 3.2.2.1
 				if ( $h->{qop} ) {
+					$h->{qop} = 'auth'; # in case it was 'auth,auth-int'
 					my $nc = $digest{nc} = '00000001';
 					my $cnonce = $digest{cnonce} = sprintf("%08x",rand(2**32));
 					$digest{qop} = $h->{qop};
