@@ -143,6 +143,17 @@ sub set_param {
 	return $self;
 }
 
+###########################################################################
+# get value for parameter(s)
+# Args: ($self,@keys)
+# Returns: @values|$value[0]
+###########################################################################
+sub get_param {
+	my Net::SIP::Simple::Call $self = shift;
+	my @v = @{$self->{param}}{@_};
+	return wantarray ? @v : $v[0];
+}
+
 ###########################################################################
 # (Re-)Invite other party
 # Args: ($self;%param)
@@ -187,7 +198,11 @@ sub reinvite {
 		my ($endpoint,$ctx,$errno,$code,$packet,$leg,$from,$ack) = @_;
 
 		if ( $errno ) {
-			$self->error( "Failed with error $errno".( $code ? " code=$code" :"" ) );
+			if ( $code == 487 ) {
+				# request was canceled, probably be me -> ignore
+			} else {
+				$self->error( "Failed with error $errno".( $code ? " code=$code" :"" ) );
+			}
 			invoke_callback( $param->{cb_final}, 'FAIL',$self,errno => $errno,code => $code,packet => $packet );
 			return;
 		}
@@ -224,6 +239,7 @@ sub reinvite {
 		invoke_callback( $param->{init_media},$self,$param );
 	};
 
+
 	my $stopvar = 0;
 	$param->{cb_final} ||= \$stopvar;
 	$cb = [ $cb,$self ];
@@ -232,15 +248,87 @@ sub reinvite {
 		$ctx, $cb, $sdp,
 		$param->{sip_header} ? %{ $param->{sip_header} } : ()
 	);
+
 	if ( $param->{cb_final} == \$stopvar ) {
-		# wait until final response
-		$self->loop( \$stopvar );
+
+		# This callback will be called on timeout or response to cancel which
+		# got send after ring_time was over
+		my $noanswercb;
+		if ( $param->{ring_time} ) {
+			$noanswercb = sub {
+				my Net::SIP::Simple::Call $self = shift || return;
+				my ($endpoint,$ctx,$errno,$code,$packet,$leg,$from,$ack) = @_;
+
+				$stopvar = 'NOANSWER' ;
+				my $param = $self->{param};
+				invoke_callback( $param->{cb_noanswer}, 'NOANSWER',$self,
+					errno => $errno,code => $code,packet => $packet );
+
+				if ( $code =~ m{^2\d\d} ) {
+					DEBUG(10,"got response of %s|%s to CANCEL",$code,$packet->msg );
+					invoke_callback( $param->{cb_final},'NOANSWER',$self,code => $code );
+				}
+			};
+			$noanswercb = [ $noanswercb,$self ];
+			weaken( $noanswercb->[1] );
+
+			# wait until final response
+			$self->loop( $param->{ring_time}, \$stopvar );
+
+			unless ($stopvar) { # timed out
+				$self->{endpoint}->cancel_invite( $self->{ctx},undef, $noanswercb );
+				$self->loop( \$stopvar );
+			}
+		} else {
+			# wait until final response
+			$self->loop( \$stopvar );
+		}
+
 		$param->{cb_final} = undef;
 	}
 	return $self->{ctx};
 }
 
 
+###########################################################################
+# cancel call
+# Args: ($self,%args)
+#   %args:
+#     cb_final: callback when CANCEL was delivered. If not given send_cancel
+#        callback on Call object will be used
+# Returns: true if call could be canceled
+# Comment: cb_final gets triggered if the reply for the CANCEL is received
+# or waiting for the reply timed out
+###########################################################################
+sub cancel {
+	my Net::SIP::Simple::Call $self = shift;
+	my %args = @_;
+
+	my $cb = delete $args{cb_final};
+	%args = ( %{ $self->{param} }, %args );
+	$cb ||= $args{send_cancel};
+
+	my $cancel_cb = [
+		sub {
+			my Net::SIP::Simple::Call $self = shift || return;
+			my ($cb,$args,$endpoint,$ctx,$error,$code) = @_;
+			# we don't care about the cause of this callback
+			# it might be a successful or failed reply packet or no reply
+			# packet at all (timeout) - the call is considered closed
+			# in any case except for 1xx responses
+			if ( $code && $code =~m{^1\d\d} ) {
+				DEBUG( 10,"got prelimary response for CANCEL" );
+				return;
+			}
+			invoke_callback( $cb,$args );
+		},
+		$self,$cb,\%args
+	];
+	weaken( $cancel_cb->[1] );
+
+	return $self->{endpoint}->cancel_invite( $self->{ctx}, undef, $cancel_cb );
+}
+
 ###########################################################################
 # end call
 # Args: ($self,%args)
@@ -449,7 +537,7 @@ sub _setup_local_rtp_socks {
 				proto => 'RTP/AVP',
 				media => 'audio',
 				fmt   => $rp->[0] || 0, # PCMU/8000
-				$rp->[3] ? ( 
+				$rp->[3] ? (
 					a => [ "rtpmap:$rp->[0] $rp->[3]" , "ptime:".$rp->[2]*1000 ]
 				) :(),
 			}
