@@ -9,7 +9,7 @@ use strict;
 use warnings;
 
 package Net::SIP::StatelessProxy;
-use fields qw( dispatcher rewrite_contact nathelper );
+use fields qw( dispatcher rewrite_contact nathelper force_rewrite );
 
 use Net::SIP::Util ':all';
 use Digest::MD5 'md5_hex';
@@ -29,6 +29,8 @@ use Net::SIP::Debug;
 #        should return undef. If not given a reasonable default will be
 #        used.
 #     nathelper: Net::SIP::NAT::Helper used for rewrite SDP bodies.. (optional)
+#     force_rewrite: if true rewrite contact even if incoming and outgoing
+#         legs are the same
 # Returns: $self
 ###########################################################################
 sub new {
@@ -40,6 +42,7 @@ sub new {
 	$self->{rewrite_contact} = delete $args{rewrite_contact}
 		|| [ \&_default_rewrite_contact, $self ];
 	$self->{nathelper} = delete $args{nathelper};
+	$self->{force_rewrite} = delete $args{force_rewrite};
 
 	return $self;
 }
@@ -61,7 +64,7 @@ sub _default_rewrite_contact {
 		my $lc = length($contact);
 		$secret = substr( $secret x int( $lc/length($secret) +1 ), 0,$lc );
 		# add 'r' in front of hex so it does not look like phone number
-		$new = 'r'.unpack( 'H*',( $contact ^ $secret )); 
+		$new = 'r'.unpack( 'H*',( $contact ^ $secret ));
 		DEBUG( 100,"rewrite $contact -> $new" );
 	} elsif ( $contact =~m{^r([0-9a-f]+)$}i ) {
 		# needs to be written back
@@ -156,10 +159,11 @@ sub __forward_response {
 	my ($addr,$port) = $first =~m{([\w\-\.]+)(?::(\d+))?\s*$};
 	$port ||= 5060; # FIXME default for sip, not sips!
 	$addr = $param->{maddr} if $param->{maddr};
+	$addr = $param->{received} if $param->{received}; # where it came from
+	$port = $param->{rport} if $param->{rport}; # where it came from
 	@{ $entry->{dst_addr}} = ( "$addr:$port" );
 	DEBUG( 50,"get dst_addr from via header: $first -> $addr:$port" );
 
-	$entry->{via_received} = $param->{received};
 	if ( $addr !~m{^[0-9\.]+$} ) {
 		$self->{dispatcher}->dns_host2ip(
 			$addr,
@@ -173,8 +177,6 @@ sub __forward_response {
 ###########################################################################
 # Called from _forward_response directly or indirectly after resolving
 # hostname of destination.
-# If received parameter was in Via header it will try to find the leg
-# based on it.
 # Calls __forward_packet_final at the end to deliver packet
 ###########################################################################
 sub __forward_response_1 {
@@ -190,29 +192,6 @@ sub __forward_response_1 {
 		$entry->{dst_addr}[0] =~s{^(udp:|tcp:)?([^:]+)}{$1$ip};
 	}
 
-	if ( my $received = $entry->{via_received} ) {
-		# FIXME: we assume that the received entry is done by us
-		# and that we only put IP addresses inside
-		my ($addr,$port) = split( ':',$received,2 );
-		my @received_legs = $self->{dispatcher}->get_legs(
-			addr => $addr, port => $port );
-		my $dst_addr = $entry->{dst_addr};
-		my @legs;
-		foreach my $addr (@$dst_addr) {
-			push @legs, grep { $_->can_deliver_to( $addr ) } @received_legs;
-		}
-
-		if ( !@legs ) {
-			# FIXME: should we really drop packet if we don't have the specified leg?
-			# or should we use any leg which could deliver to $dst_addr
-			DEBUG( 10,"cannot find leg for $received which can deliver to $dst_addr" );
-			return;
-		}
-
-		@{ $entry->{outgoing_leg} } = @legs;
-		DEBUG( 50,"getting leg from received=$received" );
-	}
-
 	__forward_packet_final( $self,$entry );
 }
 
@@ -400,7 +379,7 @@ sub __forward_packet_final {
 
 	my $packet = $entry->{packet};
 	# rewrite contact header if outgoing leg is different to incoming leg
-	if ( $outgoing_leg != $incoming_leg and 
+	if ( ( $outgoing_leg != $incoming_leg or $self->{force_rewrite} ) and
 		(my @contact = $packet->get_header( 'contact' ))) {
 
 		my $rewrite_contact = $self->{rewrite_contact};
